@@ -39,6 +39,9 @@ typedef struct
 {
   struct _gpgme_op_sign_result result;
 
+  /* The error code from a FAILURE status line or 0.  */
+  gpg_error_t failure_code;
+
   /* A pointer to the next pointer of the last invalid signer in
      the list.  This makes appending new invalid signers painless
      while preserving the order.  */
@@ -55,21 +58,8 @@ typedef struct
 
 
 static void
-release_op_data (void *hook)
+release_signatures (gpgme_new_signature_t sig)
 {
-  op_data_t opd = (op_data_t) hook;
-  gpgme_invalid_key_t invalid_signer = opd->result.invalid_signers;
-  gpgme_new_signature_t sig = opd->result.signatures;
-
-  while (invalid_signer)
-    {
-      gpgme_invalid_key_t next = invalid_signer->next;
-      if (invalid_signer->fpr)
-	free (invalid_signer->fpr);
-      free (invalid_signer);
-      invalid_signer = next;
-    }
-
   while (sig)
     {
       gpgme_new_signature_t next = sig->next;
@@ -80,12 +70,35 @@ release_op_data (void *hook)
 }
 
 
+static void
+release_op_data (void *hook)
+{
+  op_data_t opd = (op_data_t) hook;
+  gpgme_invalid_key_t invalid_signer = opd->result.invalid_signers;
+
+  while (invalid_signer)
+    {
+      gpgme_invalid_key_t next = invalid_signer->next;
+      if (invalid_signer->fpr)
+	free (invalid_signer->fpr);
+      free (invalid_signer);
+      invalid_signer = next;
+    }
+
+  release_signatures (opd->result.signatures);
+}
+
+
 gpgme_sign_result_t
 gpgme_op_sign_result (gpgme_ctx_t ctx)
 {
   void *hook;
   op_data_t opd;
   gpgme_error_t err;
+  gpgme_invalid_key_t inv_key, key;
+  gpgme_new_signature_t sig;
+  unsigned int inv_signers = 0;
+  unsigned int signatures = 0;
 
   TRACE_BEG (DEBUG_CTX, "gpgme_op_sign_result", ctx);
 
@@ -97,52 +110,85 @@ gpgme_op_sign_result (gpgme_ctx_t ctx)
       return NULL;
     }
 
-  if (_gpgme_debug_trace ())
+  for (inv_key = opd->result.invalid_signers; inv_key; inv_key = inv_key->next)
+    inv_signers++;
+  for (sig = opd->result.signatures; sig; sig = sig->next)
+    signatures++;
+
+  if (gpgme_signers_count (ctx)
+      && signatures + inv_signers != gpgme_signers_count (ctx))
     {
-      gpgme_invalid_key_t inv_key = opd->result.invalid_signers;
-      gpgme_new_signature_t sig = opd->result.signatures;
-      int inv_signers = 0;
-      int signatures = 0;
+      /* In this case at least one signatures was not created perhaps
+         due to a bad passphrase etc.  Thus the entire message is
+         broken and should not be used.  We add the already created
+         signatures to the invalid signers list and thus this case can
+         be detected.  */
+      TRACE_LOG3 ("result: invalid signers: %u, signatures: %u, count: %u",
+                  inv_signers, signatures, gpgme_signers_count (ctx));
 
-      while (inv_key)
-	{
-	  inv_signers++;
-	  inv_key = inv_key->next;
-	}
-      while (sig)
-	{
-	  signatures++;
-	  sig = sig->next;
-	}
+      for (sig = opd->result.signatures; sig; sig = sig->next)
+        {
+          key = calloc (1, sizeof *key);
+          if (!key)
+            {
+              TRACE_SUC0 ("out of core; result=(null)");
+              return NULL;
+            }
+          if (sig->fpr)
+            {
+              key->fpr = strdup (sig->fpr);
+              if (!key->fpr)
+                {
+                  free (key);
+                  TRACE_SUC0 ("out of core; result=(null)");
+                  return NULL;
+                }
+            }
+          key->reason = GPG_ERR_GENERAL;
 
+          inv_key = opd->result.invalid_signers;
+          if (inv_key)
+            {
+              for (; inv_key->next; inv_key = inv_key->next)
+                ;
+              inv_key->next = key;
+            }
+          else
+            opd->result.invalid_signers = key;
+        }
+
+      release_signatures (opd->result.signatures);
+      opd->result.signatures = NULL;
+    }
+
+  if (_gpgme_debug_trace())
+    {
       TRACE_LOG2 ("result: invalid signers: %i, signatures: %i",
 		  inv_signers, signatures);
-      inv_key = opd->result.invalid_signers;
-      while (inv_key)
+      for (inv_key=opd->result.invalid_signers; inv_key; inv_key=inv_key->next)
 	{
 	  TRACE_LOG3 ("result: invalid signer: fpr=%s, reason=%s <%s>",
 		      inv_key->fpr, gpgme_strerror (inv_key->reason),
 		      gpgme_strsource (inv_key->reason));
-	  inv_key = inv_key->next;
 	}
-      sig = opd->result.signatures;
-      while (sig)
+      for (sig = opd->result.signatures; sig; sig = sig->next)
 	{
 	  TRACE_LOG6 ("result: signature: type=%i, pubkey_algo=%i, "
 		      "hash_algo=%i, timestamp=%li, fpr=%s, sig_class=%i",
 		      sig->type, sig->pubkey_algo, sig->hash_algo,
 		      sig->timestamp, sig->fpr, sig->sig_class);
-	  sig = sig->next;
 	}
-    }
+   }
 
   TRACE_SUC1 ("result=%p", &opd->result);
   return &opd->result;
 }
 
+
 
 static gpgme_error_t
-parse_sig_created (char *args, gpgme_new_signature_t *sigp)
+parse_sig_created (char *args, gpgme_new_signature_t *sigp,
+                   gpgme_protocol_t protocol)
 {
   gpgme_new_signature_t sig;
   char *tail;
@@ -180,7 +226,7 @@ parse_sig_created (char *args, gpgme_new_signature_t *sigp)
     }
 
   gpg_err_set_errno (0);
-  sig->pubkey_algo = strtol (args, &tail, 0);
+  sig->pubkey_algo = _gpgme_map_pk_algo (strtol (args, &tail, 0), protocol);
   if (errno || args == tail || *tail != ' ')
     {
       /* The crypto backend does not behave.  */
@@ -263,7 +309,7 @@ _gpgme_sign_status_handler (void *priv, gpgme_status_code_t code, char *args)
     {
     case GPGME_STATUS_SIG_CREATED:
       opd->sig_created_seen = 1;
-      err = parse_sig_created (args, opd->last_sig_p);
+      err = parse_sig_created (args, opd->last_sig_p, ctx->protocol);
       if (err)
 	return err;
 
@@ -284,6 +330,10 @@ _gpgme_sign_status_handler (void *priv, gpgme_status_code_t code, char *args)
       opd->last_signer_p = &(*opd->last_signer_p)->next;
       break;
 
+    case GPGME_STATUS_FAILURE:
+      opd->failure_code = _gpgme_parse_failure (args);
+      break;
+
     case GPGME_STATUS_EOF:
       /* The UI server does not send information about the created
          signature.  This is irrelevant for this protocol and thus we
@@ -292,7 +342,12 @@ _gpgme_sign_status_handler (void *priv, gpgme_status_code_t code, char *args)
 	err = gpg_error (GPG_ERR_UNUSABLE_SECKEY);
       else if (!opd->sig_created_seen
                && ctx->protocol != GPGME_PROTOCOL_UISERVER)
-	err = gpg_error (GPG_ERR_GENERAL);
+	err = opd->failure_code? opd->failure_code:gpg_error (GPG_ERR_GENERAL);
+      break;
+
+    case GPGME_STATUS_INQUIRE_MAXLEN:
+      if (ctx->status_cb)
+        err = ctx->status_cb (ctx->status_cb_value, "INQUIRE_MAXLEN", args);
       break;
 
     default:
@@ -326,6 +381,7 @@ sign_init_result (gpgme_ctx_t ctx, int ignore_inv_recp)
   opd = hook;
   if (err)
     return err;
+  opd->failure_code = 0;
   opd->last_signer_p = &opd->result.invalid_signers;
   opd->last_sig_p = &opd->result.signatures;
   opd->ignore_inv_recp = !!ignore_inv_recp;
