@@ -51,6 +51,8 @@ int _gpgme_get_conf_int (const char *key, int *value);
 void _gpgme_allow_set_foreground_window (pid_t pid);
 
 /*-- dirinfo.c --*/
+void _gpgme_dirinfo_disable_gpgconf (void);
+
 const char *_gpgme_get_default_homedir (void);
 const char *_gpgme_get_default_agent_socket (void);
 const char *_gpgme_get_default_gpg_name (void);
@@ -58,6 +60,9 @@ const char *_gpgme_get_default_gpgsm_name (void);
 const char *_gpgme_get_default_g13_name (void);
 const char *_gpgme_get_default_gpgconf_name (void);
 const char *_gpgme_get_default_uisrv_socket (void);
+int _gpgme_in_gpg_one_mode (void);
+
+const char *_gpgme_get_basename (const char *name);
 
 
 
@@ -130,6 +135,8 @@ time_t _gpgme_parse_timestamp (const char *timestamp, char **endp);
 
 gpgme_error_t _gpgme_map_gnupg_error (char *err);
 
+int _gpgme_map_pk_algo (int algo, gpgme_protocol_t protocol);
+
 
 /* Retrieve the environment variable NAME and return a copy of it in a
    malloc()'ed buffer in *VALUE.  If the environment variable is not
@@ -151,25 +158,13 @@ const char *_gpgme_get_w32spawn_path (void);
 char *_gpgme_w32ce_get_debug_envvar (void);
 #endif /*HAVE_W32CE_SYSTEM*/
 
-/*--  Error codes not yet available in current gpg-error.h.   --*/
-#ifndef GPG_ERR_UNFINISHED
-#define GPG_ERR_UNFINISHED 199
-#endif
-#ifndef GPG_ERR_NOT_OPERATIONAL
-#define GPG_ERR_NOT_OPERATIONAL 176
-#endif
-#ifndef GPG_ERR_MISSING_ISSUER_CERT
-#define GPG_ERR_MISSING_ISSUER_CERT 185
-#endif
 
 
-#ifdef ENABLE_ASSUAN
 #include <assuan.h>
 /* System hooks for assuan integration.  */
 extern struct assuan_system_hooks _gpgme_assuan_system_hooks;
 extern struct assuan_malloc_hooks _gpgme_assuan_malloc_hooks;
 int _gpgme_assuan_log_cb (assuan_context_t ctx, void *hook,
 			  unsigned int cat, const char *msg);
-#endif
 
 #endif /* UTIL_H */
